unit AdvGridWithTouchSpin.MainForm;

interface

uses
  Winapi.Windows, Winapi.Messages,
  System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Grids,
  Vcl.StdCtrls, Vcl.ExtCtrls,
  AdvUtil, AdvObj, BaseGrid, AdvGrid,
  AdvMultiButtonEdit,
  AdvGridWithTouchSpin.EditLink,
  AdvGridwithTouchSpin.GridForm,
  AdvGridWithTouchSpin.BrokenGridForm;

type
  TMainForm = class(TForm)
    tseFontSize: TAdvTouchSpinEdit;
    Label1: TLabel;
    pGrid: TPanel;
    Button1: TButton;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tseFontSizeChange(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Dclarations prives }
    gridCredits: TAdvStringGrid;
    FTouchSpinEditLink: TTouchSpinEditLink;

    procedure gridCreditsCanClickCell(Sender: TObject; ARow, ACol: Integer;
      var Allow: Boolean);
    procedure gridCreditsCanEditCell(Sender: TObject; ARow, ACol: Integer;
      var CanEdit: Boolean);
    procedure gridCreditsGetAlignment(Sender: TObject; ARow, ACol: Integer;
      var HAlign: TAlignment; var VAlign: TVAlignment);
    procedure gridCreditsGetCellColor(Sender: TObject; ARow, ACol: Integer;
      AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
    procedure gridCreditsFooterPaint(Sender: TObject; AColumn: Integer;
      Canvas: TCanvas; ARect: TRect);
    procedure gridCreditsEditCellDone(Sender: TObject; ACol, ARow: Integer);
    procedure gridCreditsCellValidate(Sender: TObject; ACol, ARow: Integer;
      var Value: string; var Valid: Boolean);
    procedure gridCreditsGetEditorType(Sender: TObject; ACol, ARow: Integer;
      var AEditor: TEditorType);
    procedure gridCreditsGetEditorProp(Sender: TObject; ACol, ARow: Integer;
      AEditLink: TEditLink);

    function DPIAware(const Value: Integer): Integer;
  public
    { Dclarations publiques }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.Button1Click(Sender: TObject);
begin
  with TForm2.Create(nil) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  Form1.Show;
end;

function TMainForm.DPIAware(const Value: Integer): Integer;
begin
  Result := MulDiv(Value, Screen.PixelsPerInch, PixelsPerInch);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FTouchSpinEditLink := TTouchSpinEditLink.Create(Self);

  gridCredits := TAdvStringGrid.Create(Self);
  gridCredits.Name := 'gridCredits';
  gridCredits.Parent := pGrid;
  gridCredits.Align := alClient;
  gridCredits.AlignWithMargins := True;
  gridCredits.FloatFormat := '%m';
  gridCredits.ColCount := 5;
  gridCredits.FixedCols := 0;
  gridCredits.ColumnHeaders.Clear;
  gridCredits.ColumnHeaders.Add('Restaurant');
  gridCredits.ColumnHeaders.Add('Crdit');
  gridCredits.ColumnHeaders.Add('Disponibles');
  gridCredits.ColumnHeaders.Add('Utiliss');
  gridCredits.ColumnHeaders.Add('Total');
  gridCredits.ColWidths[4] := DPIAware(120);
  gridCredits.ColWidths[3] := DPIAware(120);
  gridCredits.ColWidths[2] := DPIAware(120);
  gridCredits.ColWidths[1] := DPIAware(120);
  gridCredits.ColumnSize.StretchColumn := 0;
  gridCredits.ColumnSize.Stretch := True;
  gridCredits.FloatingFooter.Column := 4;
  gridCredits.FloatingFooter.FooterStyle := fsCustomPaint;
  gridCredits.FloatingFooter.Visible := True;
  gridCredits.OnGetCellColor := gridCreditsGetCellColor;
  gridCredits.OnGetAlignment := gridCreditsGetAlignment;
  gridCredits.OnCanClickCell := gridCreditsCanClickCell;
  gridCredits.OnCanEditCell := gridCreditsCanEditCell;
  gridCredits.OnCellValidate := gridCreditsCellValidate;
  gridCredits.OnGetEditorType := gridCreditsGetEditorType;
  gridCredits.OnGetEditorProp := gridCreditsGetEditorProp;
  gridCredits.OnEditCellDone := gridCreditsEditCellDone;
  gridCredits.OnFooterPaint := gridCreditsFooterPaint;
  gridCredits.Options := gridCredits.Options + [goEditing];

  gridCredits.RowCount := 3;
  for var I := 1 to gridCredits.RowCount -1 do
  begin
    gridCredits.Cells[0, I]  := Format('Line %d', [I]);
    gridCredits.Floats[1, I] := 25 * I;
    gridCredits.Ints[2, I]   := gridCredits.RowCount - I;
    gridCredits.Ints[3, I]   := gridCredits.Ints[2, I];
    gridCredits.Floats[4, I] := gridCredits.Floats[1, I] * gridCredits.Ints[3, I];
  end;

  gridCredits.Col := 3;

  tseFontSize.Value := Self.Font.Size;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  gridCredits.EditLink := nil;
  FTouchSpinEditLink.Free;
end;

procedure TMainForm.gridCreditsCanClickCell(Sender: TObject; ARow, ACol: Integer;
  var Allow: Boolean);
begin
  Allow := (ARow > 0) and (ACol = 3);
end;

procedure TMainForm.gridCreditsCanEditCell(Sender: TObject; ARow, ACol: Integer;
  var CanEdit: Boolean);
begin
  CanEdit := (ARow > 0) and (ACol = 3);
end;

procedure TMainForm.gridCreditsCellValidate(Sender: TObject; ACol, ARow: Integer;
  var Value: string; var Valid: Boolean);
begin
  if (ARow > 0) and (ACol = 3) then
  begin
    var ValInt: Integer;
    Valid := TryStrToInt(Value, ValInt) and (ValInt >= 0) and (ValInt <= gridCredits.Ints[2, ARow]);
  end
  else
    Valid := False;
end;

procedure TMainForm.gridCreditsEditCellDone(Sender: TObject; ACol, ARow: Integer);
begin
  if (ARow > 0) and (ACol = 3) then
  begin
    gridCredits.Floats[4, ARow] := gridCredits.Floats[1, ARow] * gridCredits.Ints[3, ARow];
    gridCredits.UpdateFooter;
  end;
end;

procedure TMainForm.gridCreditsFooterPaint(Sender: TObject; AColumn: Integer;
  Canvas: TCanvas; ARect: TRect);
begin
  Canvas.Brush.Color := clGray;
  Canvas.FillRect(ARect);

  Canvas.Font.Color := clWhite;
  Canvas.Font.Style := [fsBold];

  var T := Format('Total %m', [gridCredits.ColumnSum(4)]);
  var R := ARect;

  Canvas.TextRect(R, T, [tfCalcRect]);
  R.Offset(ARect.Width - R.Width - DPIAware(10), (ARect.Height - R.Height) div 2);

  Canvas.TextRect(R, T);
end;

procedure TMainForm.gridCreditsGetAlignment(Sender: TObject; ARow, ACol: Integer;
  var HAlign: TAlignment; var VAlign: TVAlignment);
begin
  if ARow = 0 then
    HAlign := taCenter
  else if ACol in [1, 4] then
    HAlign := taRightJustify
  else if ACol in [2, 3] then
    HAlign := taCenter;
end;

procedure TMainForm.gridCreditsGetCellColor(Sender: TObject; ARow, ACol: Integer;
  AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
begin
  if (ARow > 0) and (ACol = 3) then
  begin
    ABrush.Color := clGray;
    AFont.Color := clWhite;
  end;
end;

procedure TMainForm.gridCreditsGetEditorProp(Sender: TObject; ACol, ARow: Integer;
  AEditLink: TEditLink);
begin
  if (ARow > 0) and (ACol = 3) then
  begin
    FTouchSpinEditLink.SpinEdit.Min := 0;
    FTouchSpinEditLink.SpinEdit.Max := gridCredits.Ints[2, ARow];
//    FTouchSpinEditLink.SpinEdit.Font.Height := gridCredits.Font.Height;
  end;
end;

procedure TMainForm.gridCreditsGetEditorType(Sender: TObject; ACol, ARow: Integer;
  var AEditor: TEditorType);
begin
  if (ARow > 0) and (ACol = 3) then
  begin
    gridCredits.EditLink := FTouchSpinEditLink;
    AEditor := edCustom;
  end;
end;

procedure TMainForm.tseFontSizeChange(Sender: TObject);
begin
  gridCredits.Font.Size := tseFontSize.Value;
  for var R := 1 to gridCredits.RowCount -1 do
    gridCredits.RowHeights[R] := Abs(gridCredits.Font.Height) + (2 * DPIAware(10));
end;

end.
